## ブートストラップパッケージのインストールとロード。
#install.packages("boot")
library(boot)

## パラメーター。
## n: 標本サイズ。
## sigma: 母集団の標準偏差。
## bootstrapCount: ブートストラップ試行数。
n <- c(10, 30, 50, 100, 500)
sigma <- 10
bootstrapCount <- c(50, 100, 500, 1000, 5000, 10000)

## 今回見たいのは分散なので，平均は何でもよい。ここでは 0 とする。
x <- lapply(n, rnorm, 0, sigma)

result <- matrix(NA, 0, length(n))
colnames(result) <- paste("n =", n)
addRow <- function(result, value, rowName)
{
	newRowName <- c(rownames(result), rowName)
	result <- rbind(result, value)
	rownames(result) <- newRowName
	return(result)
}

## 真の値。
result <- addRow(result, sigma^2/n, "Parameter")
## 不偏推定量を用いた理論値。
result <- addRow(result, sapply(1 : length(n), function(i) var(x[[i]])/n[i]), "Unbiased estimate")

## ブートストラップ法による標本平均の分散。
for (b in bootstrapCount)
{
	v <- lapply(x, boot, function(d, i) mean(d[i]), b)
	result <- addRow(result, sapply(v, function(d) var(d$t)), paste(b, "times bootstrapping"))
}

print(result)
