## ブートストラップパッケージのインストールとロード。
#install.packages("boot")
library(boot)
library(MASS)

## help(glm) から。データの出典は McCullagh & Nelder (1989)。
## u: 血漿濃度 [%]。
## lot1: 凝固時間 [sec]。
u <- c(5, 10, 15, 20, 30, 40, 60, 80, 100)
lot1 <- c(118, 58, 42, 35, 27, 25, 21, 19, 18)

bootstrapCount <- 1000
errorRate <- 0.05

model <- glm(lot1 ~ log(u), family = Gamma)
shape <- gamma.shape(model)$alpha
scale <- model$fitted.values / shape

## 回帰曲線と，各データのばらつきを示す。
x <- seq(1.5, 5, length.out = 300)
plot(model$formula, xlim = c(1.5, 5), ylim = c(0, 140), xlab = "Log(concentration) [log(%)]", ylab = "Time [sec]")
lines(x, model$family$linkfun(model$coefficients[1] + model$coefficients[2] * x))
for (i in 1 : length(u))
{
	lower <- qgamma(errorRate/2, shape = shape, scale = scale[i])
	upper <- qgamma(1 - errorRate/2, shape = shape, scale = scale[i])
	y <- seq(lower, upper, length.out = 100)
	d <- dgamma(y, shape = shape, scale = scale[i]) + log(u[i])
	lines(d, y, col = rgb(0.3, 0.8, 0.3))
}

## ブートストラップ。
boot.out <- boot(lot1, function(d) glm(d ~ log(u), family = Gamma)$coefficients, bootstrapCount, "parametric",
                 ran.gen = function(d, p) rgamma(p$n, p$shape, scale = p$scale),
                 mle = list(n = length(u), shape = shape, scale = scale))
## パーセンタイル法による信頼区間の計算。
## ちなみに推定量 (boot 関数の第 2 引数) がスカラーの場合は boot.ci 関数でパーセンタイル法以外の方法でも計算してくれる。
interval <- apply(boot.out$t, 2, quantile, c(errorRate/2, 1 - errorRate/2))
colnames(interval) <- names(boot.out$t0)

interval <- rbind(matrix(model$coefficients, 1, length(model$coefficients)), interval)
rownames(interval) <- c("Model", "Lower", "Upper")

print(interval)
